@@ -138,7 +138,7 @@ liq_error_stuff[] = {
     {LIQ_FREED_POINTER, "bad access to freed pointer"},
     {LIQ_NULL_POINTER, "null pointer"},
     {LIQ_BAD_POINTER, "pointer with wrong type"},
-    {LIQ_NEGATIVE_DIMENSIONS, "image dimensions are negative"},
+    {LIQ_NEGATIVE_DIMENSIONS, "image dimensions are zero or negative"},
     {LIQ_HUGE_DIMENSIONS, "image dimensions are unfeasible"},
     {LIQ_BAD_USER_POINTER, "liq hates your pointer"},
 };
@@ -263,10 +263,12 @@ image_quantize_png_data_to_rgba (image_quantize_t * iq)
     iq_png_t * p;
     png_bytepp row_pointers;
     liq_error error;
+    png_uint_32 width;
+    png_uint_32 height;
 
     p = iq->hook;
 
-    png_get_IHDR (p->in_png, p->in_info, & iq->width, & iq->height,
+    png_get_IHDR (p->in_png, p->in_info, & width, & height,
 		  & bit_depth, & colour_type, 0, 0, 0);
     if (bit_depth != 8) {
 	image_quantize_perl_error (iq, "bit depth %d cannot be handled yet",
@@ -276,6 +278,18 @@ image_quantize_png_data_to_rgba (image_quantize_t * iq)
 	image_quantize_perl_error (iq, "colour type %d cannot be handled yet",
 				   colour_type);
     }
+    if (width <= 0) {
+	image_quantize_perl_error (iq, "image has negative or zero width %d",
+				   width);
+    }
+    iq->width = width;
+
+    if (height <= 0) {
+	image_quantize_perl_error (iq, "image has negative or zero height %d",
+				   height);
+    }
+    iq->height = height;
+
     row_pointers = png_get_rows (p->in_png, p->in_info);
     error = liq_image_create_rgba_rows_2 (iq->attr, (void **) row_pointers,
 					  iq->width, iq->height, 0,
